;  Test SWCHB
;  Written by Omegamatrix
;
;
;*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
;                                                                                                ;
;                                     OMEGA REVISION 1.1                                         ;
;                                                                                                ;
;*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
;
;
;  This program tests if you can actual use the extra bits in SWCHB to store data as suggested here:
;
;  http://www.biglist.com/lists/stella/archives/200407/msg00042.html
;
;
;
;  The original demo I wrote failed. My mistake was in setting the directional register back to
;  input to read SWCHB. For this to work you leave the 3 unused pins as output all the time. Then
;  you can read SWCHB as you normally would. The 3 unused pins retain the state of the previous
;  write.
;
;
;  Note:
;  This program has passed on both a 7800 and Atari Jr, but fails in Stella 3.1 as of Aug 6, 2010.
;
;
;  SWCHB bit map:
;
;      7 6 5 4 3 2 1 0
;      | | | | | | | |
;      | | | | | | | Reset (0 = pressed)
;      | | | | | | Select (0 = pressed)
;      | | | | | |
;      | | | | | Not used
;      | | | | Color/B&W (0 = black and white, 1 = color)
;      | | | |
;      | | | Not used
;      | | Not used
;      | |
;      | P0 difficulty (0 = amateur, 1 = pro)
;     P1 difficulty (0 = amateur, 1 = pro)
;
;  SWCHB holds the state of the console switches on the Atari 2600. There are 3 unused Data bits (D2,D4, and D5).
;  To use these data bits first write #$34 to SWBCNT, which is the data direction register (DDR) for SWCHB. This
;  Will keep all the console switches as outputs and the unused bits as outputs. Never update SWBCNT again after
;  this. You can now read and write values to SWCHB like any other register. The writing will only affect the
;  unused pins, because the switches are configured as inputs. When you read you get the value of the switches
;  plus the 3 unused bits.
;
;
;  Have fun!! :)


      processor 6502

VSYNC   =  $00
VBLANK  =  $01
WSYNC   =  $02
NUSIZ0  =  $04
NUSIZ1  =  $05
COLUP0  =  $06
COLUP1  =  $07
RESP0   =  $10
RESP1   =  $11
GRP0    =  $1B
GRP1    =  $1C
HMP0    =  $20
HMP1    =  $21
VDELP0  =  $25
VDELP1  =  $26
HMOVE   =  $2A
INTIM   =  $0284
TIM64T  =  $0296
SWCHB   =  $0282
SWBCNT  =  $0283

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      RAM
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       SEG.U Variables
       ORG $80

index            ds   1
counter          ds   1
frameCounter     ds   1
gfxPtrs          ds   12
loopCount        ds   1
tempOne          ds   1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      MAIN PROGRAM
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       SEG Code
       ORG $F000

START:
    sei
    cld
    ldx    #0
    txa
.loopClear:
    dex
    txs
    pha
    bne    .loopClear

;initialization
    sta   WSYNC
;---------------------------------------
    lda    #$0F                  ;2  @2     white
    sta    COLUP0                ;3  @5
    sta    COLUP1                ;3  @8
    sta    VDELP0                ;3  @11    delay
    sta    VDELP1                ;3  @14
    lda    #$03                  ;2  @16
    sta    NUSIZ0                ;3  @29    3 copies close
    sta    NUSIZ1                ;3  @22
    lda    #$80                  ;2  @24    right 8
    sta    HMP0                  ;3  @27
    lda    #$90                  ;2  @29    right 7
    sta    HMP1                  ;3  @32
    nop                          ;2  @34
    sta    RESP0                 ;3  @37
    sta    RESP1                 ;3  @40


;counter ram location is zero from clear loop!!

    sta    WSYNC                 ;3  @76
;---------------------------------------
    sta    HMOVE


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      VBLANK
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

MainLoop:
    lda    #$0E                     ; does three lines of Vsync
.loopVsync:
    sta    WSYNC
;---------------------------------------
    sta    VSYNC
    lsr
    bne    .loopVsync
    lda    #$31
    sta    TIM64T

;write to SWCHB
    lda    #$34                     ; 0011 0100   output on pins D5, D4, and D2
    sta    SWBCNT                   ; (1's set individual DDR bits to output, and 0's are for input)
    lda    counter
    sta    SWCHB

;display number written to SWCHB
    ldx    #gfxPtrs+11
    txs
    ldx    #5
.loopWritePtrs:
    lda    #>WriteOne
    pha
    lda    WritePtrs,X
    pha
    dex
    bpl    .loopWritePtrs

    txs
    lax    counter
    and    #$30
    lsr
    sta    gfxPtrs+8
    txa
    and    #$04
    asl
    asl
    asl
    sta    gfxPtrs+10

.waitVblank:
    lda    INTIM
    bne    .waitVblank
    sta    WSYNC
;---------------------------------------
    sta    VBLANK                   ; Vblank ends
    sta    WSYNC                    ; buffer

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      KERNEL
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ldy    #78
    jsr    DelayScanlines
    jsr    SixByteDisplay

;read SWCHB
    ldx    #gfxPtrs+11
    txs
    ldx    #5
.loopReadPtrs:
    lda    #>ReadOne
    pha
    lda    ReadPtrs,X
    pha
    dex
    bpl    .loopReadPtrs

    txs

;**************************************************************************************************
;
;  The original test I did failed. My mistake was setting the directional register back to input to
;  read it. For this to work you leave those 3 pins as output all the time. You just read SWCHB as
;  you normally would, and those 3 pins will still retain the state of the previous write.
;
;**************************************************************************************************

;No longer set the direction back to input!!
;    lda    #0                       ; all input
;    sta    SWBCNT                   ;

    lda    SWCHB                     ; those 3 pins are output, the switches are all input
    and    #$34
    tax
    and    #$30
    lsr
    sta    gfxPtrs+8
    txa
    and    #$04
    asl
    asl
    asl
    sta    gfxPtrs+10

    ldy    #10
    jsr    DelayScanlines
    jsr    SixByteDisplay
    ldy    #85
    jsr    DelayScanlines

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      OVERSCAN
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    lda    #$82
    sta    WSYNC
;---------------------------------------
    sta    VBLANK                   ; turn beam off, start overscan period
    lda    #26
    sta    TIM64T

    inc    frameCounter
    lda    frameCounter
    cmp    #$50
    bne    .waitOverscan
    lda    #0
    sta    frameCounter
    inc    index
    ldx    index
    cpx    #8
    bne    .skipIndexReset
    cpx    #8
    tax                             ; X=0
    stx    index
.skipIndexReset:
    lda    BitTab,X
    sta    counter


.waitOverscan:
    lda    INTIM
    bne    .waitOverscan
    sta    WSYNC                    ; buffer
    sta    WSYNC                    ; buffer
    jmp    MainLoop



       ORG $FE00

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SixByteDisplay SUBROUTINE
    ldy    #7
    sty    loopCount
.loopDrawGfx:
    ldy    loopCount             ;3  @64
    lda    (gfxPtrs+10),Y        ;5  @69
    sta    tempOne               ;3  @72    6th
    sta    WSYNC                 ;3         1 cycle to spare
;---------------------------------------
    lda    (gfxPtrs),Y           ;5  @5     1st
    sta    GRP0                  ;3  @8
    lda    (gfxPtrs+2),Y         ;5  @13    2nd
    sta    GRP1                  ;3  @16
    lda    (gfxPtrs+4),Y         ;5  @21    3rd
    sta    GRP0                  ;3  @24
    lda    (gfxPtrs+8),Y         ;5  @29    5th
    tax                          ;2  @31
    lda    (gfxPtrs+6),Y         ;5  @36    4th
    ldy    tempOne               ;3  @39    6th
    nop                          ;2  @41
    sta    GRP1                  ;3  @44
    stx    GRP0                  ;3  @47
    sty    GRP1                  ;3  @50
    sta    GRP0                  ;3  @53
    dec    loopCount             ;5  @58
    bpl    .loopDrawGfx          ;2 @60/61
    lda    #0                    ;2  @62
    sta    GRP0                  ;3  @65
    sta    GRP1                  ;3  @68
    sta    GRP0                  ;3  @71
    sta    GRP1                  ;3  @74
;---------------------------------------
    rts                          ;6  @4

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DelayScanlines SUBROUTINE
.loopDelay:
    sta    WSYNC
;---------------------------------------
    dey                          ;2  @2
    bne    .loopDelay            ;2 @4/5
    rts                          ;6  @10

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       ORG $FF00

Zero:
    .byte $3C ; |  XXXX  | $FE00
    .byte $66 ; | XX  XX | $FF01
    .byte $66 ; | XX  XX | $FF02
    .byte $66 ; | XX  XX | $FF03
    .byte $66 ; | XX  XX | $FF04
    .byte $66 ; | XX  XX | $FF05
    .byte $66 ; | XX  XX | $FF06
    .byte $3C ; |  XXXX  | $FF07
One:
    .byte $3C ; |  XXXX  | $FF08
    .byte $18 ; |   XX   | $FF09
    .byte $18 ; |   XX   | $FF0A
    .byte $18 ; |   XX   | $FF0B
    .byte $18 ; |   XX   | $FF0C
    .byte $18 ; |   XX   | $FF0D
    .byte $38 ; |  XXX   | $FF0E
    .byte $18 ; |   XX   | $FF0F
Two:
    .byte $7E ; | XXXXXX | $FF10
    .byte $60 ; | XX     | $FF11
    .byte $60 ; | XX     | $FF12
    .byte $3C ; |  XXXX  | $FF13
    .byte $06 ; |     XX | $FF14
    .byte $06 ; |     XX | $FF15
    .byte $46 ; | X   XX | $FF16
    .byte $3C ; |  XXXX  | $FF17
Three:
    .byte $3C ; |  XXXX  | $FF18
    .byte $46 ; | X   XX | $FF19
    .byte $06 ; |     XX | $FF1A
    .byte $0C ; |    XX  | $FF1B
    .byte $0C ; |    XX  | $FF1C
    .byte $06 ; |     XX | $FF1D
    .byte $46 ; | X   XX | $FF1E
    .byte $3C ; |  XXXX  | $FF1F
Four:
    .byte $0C ; |    XX  | $FF20
    .byte $0C ; |    XX  | $FF21
    .byte $0C ; |    XX  | $FF22
    .byte $7E ; | XXXXXX | $FF23
    .byte $4C ; | X  XX  | $FF24
    .byte $2C ; |  X XX  | $FF25
    .byte $1C ; |   XXX  | $FF26
    .byte $0C ; |    XX  | $FF27
Five:
    .byte $7C ; | XXXXX  | $FF28
    .byte $46 ; | X   XX | $FF29
    .byte $06 ; |     XX | $FF2A
    .byte $06 ; |     XX | $FF2B
    .byte $7C ; | XXXXX  | $FF2C
    .byte $60 ; | XX     | $FF2D
    .byte $60 ; | XX     | $FF2E
    .byte $7E ; | XXXXXX | $FF2F
Six:
    .byte $3C ; |  XXXX  | $FF30
    .byte $66 ; | XX  XX | $FF31
    .byte $66 ; | XX  XX | $FF32
    .byte $66 ; | XX  XX | $FF33
    .byte $7C ; | XXXXX  | $FF34
    .byte $60 ; | XX     | $FF35
    .byte $62 ; | XX   X | $FF36
    .byte $3C ; |  XXXX  | $FF37
Seven:
    .byte $18 ; |   XX   | $FF38
    .byte $18 ; |   XX   | $FF39
    .byte $18 ; |   XX   | $FF3A
    .byte $18 ; |   XX   | $FF3B
    .byte $0C ; |    XX  | $FF3C
    .byte $06 ; |     XX | $FF3D
    .byte $42 ; | X    X | $FF3E
    .byte $7E ; | XXXXXX | $FF3F
Eight:
    .byte $3C ; |  XXXX  | $FF40
    .byte $66 ; | XX  XX | $FF41
    .byte $66 ; | XX  XX | $FF42
    .byte $3C ; |  XXXX  | $FF43
    .byte $3C ; |  XXXX  | $FF44
    .byte $66 ; | XX  XX | $FF45
    .byte $66 ; | XX  XX | $FF46
    .byte $3C ; |  XXXX  | $FF47
Nine:
    .byte $3C ; |  XXXX  | $FF48
    .byte $46 ; | X   XX | $FF49
    .byte $06 ; |     XX | $FF4A
    .byte $3E ; |  XXXXX | $FF4B
    .byte $66 ; | XX  XX | $FF4C
    .byte $66 ; | XX  XX | $FF4D
    .byte $66 ; | XX  XX | $FF4E
    .byte $3C ; |  XXXX  | $FF4F
BlankDigit:
    .byte $00 ; |        | $FF50
    .byte $00 ; |        | $FF51
    .byte $00 ; |        | $FF52
    .byte $00 ; |        | $FF53
    .byte $00 ; |        | $FF54
    .byte $00 ; |        | $FF55
    .byte $00 ; |        | $FF56
    .byte $00 ; |        | $FF57


ReadOne:
    .byte $00 ; |        | $FF58
    .byte $45 ; | X   X X| $FF59
    .byte $45 ; | X   X X| $FF5A
    .byte $4D ; | X  XX X| $FF5B
    .byte $79 ; | XXXX  X| $FF5C
    .byte $45 ; | X   X X| $FF5D
    .byte $45 ; | X   X X| $FF5E
    .byte $79 ; | XXXX  X| $FF5F
ReadTwo:
    .byte $00 ; |        | $FF60
    .byte $E8 ; |XXX X   | $FF61
    .byte $08 ; |    X   | $FF62
    .byte $08 ; |    X   | $FF63
    .byte $CF ; |XX  XXXX| $FF64
    .byte $08 ; |    X   | $FF65
    .byte $08 ; |    X   | $FF66
    .byte $E7 ; |XXX  XXX| $FF67
ReadThree:
    .byte $00 ; |        | $FF68
    .byte $BC ; |X XXXX  | $FF69
    .byte $A2 ; |X X   X | $FF6A
    .byte $A2 ; |X X   X | $FF6B
    .byte $A2 ; |X X   X | $FF6C
    .byte $A2 ; |X X   X | $FF6D
    .byte $A2 ; |X X   X | $FF6E
    .byte $3C ; |  XXXX  | $FF6F
WriteOne:
    .byte $00 ; |        | $FF70
    .byte $72 ; | XXX  X | $FF71
    .byte $AA ; |X X X X | $FF72
    .byte $AA ; |X X X X | $FF73
    .byte $AB ; |X X X XX| $FF74
    .byte $8A ; |X   X X | $FF75
    .byte $8A ; |X   X X | $FF76
    .byte $8B ; |X   X XX| $FF77
WriteTwo:
    .byte $00 ; |        | $FF78
    .byte $28 ; |  X X   | $FF79
    .byte $28 ; |  X X   | $FF7A
    .byte $68 ; | XX X   | $FF7B
    .byte $C8 ; |XX  X   | $FF7C
    .byte $28 ; |  X X   | $FF7D
    .byte $28 ; |  X X   | $FF7E
    .byte $CB ; |XX  X XX| $FF7F
WriteThree:
    .byte $00 ; |        | $FF80
    .byte $8F ; |X   XXXX| $FF81
    .byte $88 ; |X   X   | $FF82
    .byte $88 ; |X   X   | $FF83
    .byte $8E ; |X   XXX | $FF84
    .byte $88 ; |X   X   | $FF85
    .byte $88 ; |X   X   | $FF86
    .byte $EF ; |XXX XXXX| $FF87

ReadPtrs:
    .byte <ReadOne       ; $FF88
    .byte <ReadTwo       ; $FF89
    .byte <ReadThree     ; $FF8A
    .byte <BlankDigit    ; $FF8B
    .byte <BlankDigit    ; $FF8C
    .byte <BlankDigit    ; $FF8D

WritePtrs:
    .byte <WriteOne      ; $FF8E
    .byte <WriteTwo      ; $FF8F
    .byte <WriteThree    ; $FF90
    .byte <BlankDigit    ; $FF91
    .byte <BlankDigit    ; $FF92
    .byte <BlankDigit    ; $FF93

BitTab:
    .byte $00            ; $FF94
    .byte $04            ; $FF95
    .byte $10            ; $FF96
    .byte $14            ; $FF97
    .byte $20            ; $FF98
    .byte $24            ; $FF99
    .byte $30            ; $FF9A
    .byte $34            ; $FF9B



       ORG $FFFC

    .word START          ; reset
    .word START          ; break